import base64
import cv2
import io
import numpy as np
import re
from datetime import datetime
from PIL import Image
from bankstatementextractor.bank_utils import *

import fitz  # PyMuPDF library
import PyPDF2  # PyPDF2 library
import os
import subprocess
import torch
from pdf2image import convert_from_path
from unidecode import unidecode
import pandas as pd
import itertools    
os.sys.path
from io import StringIO

class Banks:

    def __init__(self):
        pass

    # def adcb_1(): 
    def adib_1(self,file_path):
        try:
            doc = fitz.open(file_path)
            output = []

            for page in doc:
                page_output = page.get_text("blocks")
                output.append(page_output)

            plain_text_data = []

            for page_output in output:
                previous_block_id = 0
                page_plain_text_data = []

                for block in page_output:
                    if block[6] == 0:
                        if previous_block_id != block[5]:
                            plain_text = unidecode(block[4])
                            page_plain_text_data.append(plain_text)
                            previous_block_id = block[5]

                if 'Consolidated Statement\n' in page_plain_text_data:
                    continue

                page_plain_text_data = [text for text in page_plain_text_data if not text.startswith(('balance', 'opening'))]

                plain_text_data.append(page_plain_text_data)

            flat_data = [item for sublist in plain_text_data for item in sublist]

            ob = r'Opening Balance\n([\d,.]+)'
            opening_balance = None

            for item in flat_data:
                match = re.search(ob, item)
                if match:
                    opening_balance = match.group(1)
                    opening_balance = float(opening_balance.replace(',', ''))
                    break

            cb = r'Closing Balance \n([\d,.]+)'
            closing_balance = None

            for item in flat_data:
                match = re.search(cb, item)
                if match:
                    closing_balance = match.group(1)
                    closing_balance = float(closing_balance.replace(',', ''))
                    break

            pattern = r'(\d{2}-\d{2}-\d{4})\s+([\s\S]*?)\n([\d,]+\.\d{2})\n([\d,]+\.\d{2})'
            matches = re.findall(pattern, '\n'.join(flat_data))
            data_list = []
            previous_amount2 = None

            for match in matches:
                date = match[0]
                description = match[1]
                amount1 = match[2]
                amount2 = match[3]

                amount2_numeric = float(amount2.replace(',', ''))

                if previous_amount2 is not None and amount2_numeric < previous_amount2:
                    amount1 = '-' + amount1

                previous_amount2 = amount2_numeric

                data_list.append((date, description, amount1, amount2))

            columns = ["timestamp", "description", "amount", "running_balance"]
            df = pd.DataFrame(data_list, columns=columns)
            df['amount'] = df['amount'].apply(lambda x: float(x.replace(',', '')))
            df['running_balance'] = df['running_balance'].apply(lambda x: float(x.replace(',', '')))
            df['timestamp'] = pd.to_datetime(df['timestamp'], format='%d-%m-%Y')
            print(df)
            if df.loc[0, 'running_balance'] < opening_balance:
                df.loc[0, 'amount'] = -df.loc[0, 'amount']

            json_data = df.to_json(orient='records')
            with open('data_1.json', 'w') as json_file:
                json_file.write(json_data)

            doc.close()

            return json_file  # Indicate success

        except Exception as e:
            return str(e)  # Return the error message if an exception occurs

        
    # def adib_2():
    def cbd_1(self,file_path):
        try:
            pattern = r'^\d{2}/\d{2}/\d{4}'  # Pattern to check if the transaction starts with a date
            
            count = 0
            name = ''
            currency = ''
            acc_type = ''
            iban = ''
            account_number = ''
            branch = ''
            
            doc = fitz.open(file_path)
            output = []
            data_ = []
            prev_balance = 0
            
            for page in doc:
                output += page.get_text("blocks")

            for tup in output:
                trans = tup[4].encode("ascii", errors="ignore").decode()

                # TODO: extracting account details
                if count == 2:
                    if trans.startswith('::\n'):
                        branch = output[count+1][4].replace('\n',',').rstrip(',')
                        name = output[count+2][4].strip().split('\n')[1].strip()
                    else:
                        branch = trans.replace('\n',',').rstrip(',')
                        name = output[count+1][4].strip().split('\n')[1].strip()

                if trans.lower().startswith('acct. no.') and account_number == '':
                    account_number = trans.strip().split('\n')[1].strip()

                if trans.lower().startswith('iban') and iban == '':
                    iban = trans.strip().split('\n')[1].strip()
                    acc_type = trans.strip().split('\n')[-1].strip()

                if trans.strip().lower().startswith('currency') and currency == '':
                    currency = trans.strip().split('\n')[1]
                    if "-" in currency:
                        currency = currency[currency.index('-')+1:].strip()

                if trans.strip().startswith('Balance Brought FWD'):
                    prev_balance = trans.split('\n')[1].replace(',','')

                # TODO: extracting transaction details
                if re.match(pattern, trans):
                    trans_list = trans.split('\n')
                    t_date = trans_list[0]

                    # TODO: extraction of description - number of lines = 1
                    if len(trans_list) == 6:
                        description = trans_list[1]
                        cur_balance = trans_list[4].replace(',','')

                        # check if the transaction is debit or credit
                        if float(cur_balance) > float(prev_balance):
                            amount = trans_list[3]
                        else:
                            amount = '-' + trans_list[3]

                    # TODO: extraction of description - number of lines = 2
                    elif len(trans_list) == 7:
                        description = trans_list[1] + ' ' + trans_list[2]
                        cur_balance = trans_list[5].replace(',','')

                        # check if the transaction is debit or credit
                        if float(cur_balance) > float(prev_balance):
                            amount = trans_list[4]
                        else:
                            amount = '-' + trans_list[4]

                    # TODO: extraction of description - number of lines = 3
                    elif len(trans_list) == 8:
                        description = trans_list[1] + ' ' + trans_list[2] + ' ' + trans_list[3] 
                        cur_balance = trans_list[6].replace(',','')

                        # check if the transaction is debit or credit
                        if float(cur_balance) > float(prev_balance):
                            amount = trans_list[5]
                        else:
                            amount = '-' + trans_list[5]

                    # TODO: extraction of description - number of lines = 4
                    elif len(trans_list) == 9:
                        description = trans_list[1] + ' ' + trans_list[2] + ' ' + trans_list[3] + ' ' + trans_list[4]
                        cur_balance = trans_list[7].replace(',','')

                        # check if the transaction is debit or credit
                        if float(cur_balance) > float(prev_balance):
                            amount = trans_list[6]
                        else:
                            amount = '-' + trans_list[6]

                    # TODO: extraction of description - number of lines = 5
                    elif len(trans_list) == 10:
                        description = trans_list[1] + ' ' + trans_list[2] + ' ' + trans_list[3] + ' ' + trans_list[4] + ' ' + trans_list[5]
                        cur_balance = trans_list[8].replace(',','')

                        # check if the transaction is debit or credit
                        if float(cur_balance) > float(prev_balance):
                            amount = trans_list[7]
                        else:
                            amount = '-' + trans_list[7]

                    obj = {
    #                         'name' : name,
    #                         'currency_code' : currency,
    #                         'type' : acc_type,
    #                         'iban' : iban,
    #                         'account_number' : account_number,
    #                         'bank_name' : 'Commercial Bank of Dubai',
    #                         'branch' : branch,
                            "timestamp": t_date,  # Keep it as a string
                            "description": description,
                            "amount": amount,
                            "running_balance": cur_balance
                        }
                    data_.append(obj)

                    prev_balance = cur_balance

                count += 1

            df2 = pd.DataFrame(data_)
        
            # Convert the timestamp column to datetime
            df["timestamp"] = pd.to_datetime(df["timestamp"], format='%d/%m/%Y')
            df['amount'] = df['amount'].apply(lambda x: float(x.replace(',', '')))
            df['running_balance'] = df['running_balance'].apply(lambda x: float(x.replace(',', '')))
            # Print the JSON
            json_data = json.dumps(data_, indent=4)
            #print("JSON Data:\n", json_data)

            # Print the DataFrame
            print(df)
            return json_data

        except Exception as e:
            return str(e)
        

    # def ei_1():    
    # def fab_1():       
    # def enbd_1():
    def liv_1(self,file_path):
        try:
            doc = fitz.open(file_path)
            output = []
            for page in doc:
                page_output = page.get_text("blocks")  # Get the text blocks for each page
                output.append(page_output)  # Append the page output to the main output list

            plain_text_data = []  # Initialize an empty list to store the plain text

            for page_output in output:
                previous_block_id = 0  # Set a variable to mark the block id
                page_plain_text_data = []  # Initialize an empty list to store the plain text for each page

                for block in page_output:
                    if block[6] == 0:  # We only take the text
                        if previous_block_id != block[5]:  # Compare the block number
                            plain_text = unidecode(block[4])
                            page_plain_text_data.append(plain_text)  # Store the plain text in the list for the current page
                            previous_block_id = block[5]  # Update the previous block id

                # Check if 'Consolidated Statement' is present in the sublist and delete the sublist
                if 'Consolidated Statement\n' in page_plain_text_data:
                    continue  # Skip the current page 
                plain_text_data.append(page_plain_text_data)

            # Initialize variables to store information
            name = None
            address = None
            statement_period = None
            account_balance = None
            account_no = None
            iban = None

            # Iterate through the list and search for the respective information
            for item in plain_text_data[0]:
                if item.startswith('Name :'):
                    name = item.split('Name :')[1].replace('\n', '')
                elif item.startswith('Address :'):
                    address = item.split('Address :')[1].replace('\n', '')
                elif item.startswith('Statement period :'):
                    statement_period = item.split('Statement period :')[1].replace('\n', '')
                elif item.startswith('Account balance :'):
                    account_balance = item.split('Account balance :')[1].replace('\n', '')
                elif item.startswith('AccountNo :'):
                    account_no = item.split('AccountNo :')[1].replace('\n', '')
                elif item.startswith('IBAN :'):
                    iban = item.split('IBAN :')[1].replace('\n', '')

            obj = {
                'name': name,
                'currency_code': 'AED',
                'type': '',
                'iban': iban,
                'account_number': account_no,
                'bank_name': 'LIV by ENBD',
                'branch': '',
                'address': address,
                'account_balance': account_balance
            }

            opening_balance_index = next((i for sublist in plain_text_data for i, text in enumerate(sublist) if text.startswith('IBAN :')), -1)

            if opening_balance_index != -1:
                plain_text_data = plain_text_data[opening_balance_index + 1:]

            plain_text_data = [[element for element in sublist if not element.startswith(('Confirmation of the correctness', 'This statement is generated on', 'Money in'))] for sublist in plain_text_data]
            plain_text_data = list(itertools.chain.from_iterable(plain_text_data))
            plain_text_data = [element.rstrip() for element in plain_text_data]
            plain_text_data = [elem.split('\n') for elem in plain_text_data]

            # Initialize empty lists to store data
            dates = []
            transactions = []
            amounts = []
            balances = []

            # Regex pattern to match date'dd/mm/yyyy'
            date_pattern = r'\d{2}/\d{2}/\d{4}'

            # Initialize a transaction variable to concatenate transaction details
            transaction = ""

            # Iterate through the list
            for item in plain_text_data:
                if re.match(date_pattern, item[0]):
                    # If the item is a date, store it and reset the transaction variable
                    date = item[0]
                    transaction = ""
                else:
                    # Concatenate the elements in the sublist after the date to form the transaction
                    transaction += " ".join(item) + " "

                    # Check if the item contains an amount and balance
                    amount_match = re.search(r'([-+]?\d{1,3}(?:,\d{3})*\.\d+)', item[0])
                    if amount_match:
                        amount = amount_match.group(1).replace(',', '')  # Remove commas
                        balance = item[-1].replace(',', '')  # Remove commas
                        # Append data to respective lists
                        dates.append(pd.to_datetime(date, format='%d/%m/%Y'))
                        transactions.append(transaction.strip())  # Remove trailing spaces
                        amounts.append(pd.to_numeric(amount))
                        balances.append(pd.to_numeric(balance))

            # Create a DataFrame
            df = pd.DataFrame({'timestamp': dates, 'description': transactions, 'amount': amounts, 'running_balance': balances})
            print(df)
            # Input json
            combined_data = {'personal_data': obj, 'transactions': json.loads(df.to_json(orient='records'))}
            combined_json = json.dumps(combined_data, indent=4)

            # Close the PDF document
            doc.close()

            return combined_json

        except Exception as e:
            return str(e)


    # def mashreq_1():    
    # def sib_1():


import base64
import cv2
import io
import numpy as np
import re
from datetime import datetime
from PIL import Image

import fitz  # PyMuPDF library
import PyPDF2  # PyPDF2 library
import os
import subprocess
import torch
from pdf2image import convert_from_path
from unidecode import unidecode
import pandas as pd
import itertools    
os.sys.path
from io import StringIO


def is_pdf(file_path):
    try:
        with open(file_path, 'rb') as file:
            pdf_header = file.read(4)
        return pdf_header == b'%PDF'
    except Exception as e:
        print("Error in is_pdf:", e)
        return False


import base64
import cv2
import io
import numpy as np
import re
from datetime import datetime
from PIL import Image
from bankstatementextractor.bank_utils import *
from bankstatementextractor.banks import Banks
from pdf2image import convert_from_bytes
import fitz  # PyMuPDF library
import PyPDF2  # PyPDF2 library
import os
import subprocess
import torch
from pdf2image import convert_from_path
from unidecode import unidecode
import pandas as pd
import itertools    
os.sys.path
from io import StringIO
from io import BytesIO
import pkg_resources


class BankExtractor:

    def __init__(self):
        """
        This is the initialization function of a class that imports a spoof model and loads an BS
        extractor.
        """
        self.bank_labels = ["adib_1","cbd_1","liv_1"]
    #     self.bank_labels = {
    # #                 "adcb_1": extract_adcb_1,
    # #                 "adcb_2": extract_adcb_2,
    # #                 "adcb_3": extract_adcb_3,
    # #                 "adcb_4": extract_adcb_4,
    # #                 "adcb_5": extract_adcb_5,
    # #                 "adcb_6": extract_adcb_6,
    #                 "adib_1": extract_adib_1,
    # #                 "adib_2": extract_adib_2,
    # #                 "adib_3": extract_adib_3,
    # #                 "ajman_1": extract_ajman_1,
    # #                 "ajman_2": extract_ajman_2,
    # #                 "ajman_3": extract_ajman_3,
    # #                 "alhilal_1":extract_alhilal_1,
    # #                 "alhilal_2":extract_alhilal_2,
    # #                 "alhilal_3":extract_alhilal_3,
    #                 "cbd_1": extract_cbd_1,
    # #                 "citi_1": extract_citi_1,
    # #                 "dib_1": extract_dib_1,
    # #                 "ei_1": extract_ei_1,
    # #                 "ei_2": extract_ei_2,
    # #                 "ei_3": extract_ei_3,
    # #                 "ei_4": extract_ei_4,
    # #                 "enbd_1": extract_enbd_1,
    # #                 "enbd_2": extract_enbd_2,
    # #                 "enbd_3": extract_enbd_3,
    # #                 "enbd_4": extract_enbd_4,
    # #                 "fab_1": extract_fab_1,
    # #                 "fab_2": extract_fab_2,
    # #                 "fab_3": extract_fab_3,
    # #                 "hsbc_1": extract_hsbc_1,
    # #                 "hsbc_2": extract_hsbc_2,
    #                 "liv_1": extract_liv_1,
    # #                 "mashreq_1": extract_mashreq_1,
    # #                 "mashreq_2": extract_mashreq_2,
    # #                 "mashreq_3": extract_mashreq_3,
    # #                 "rak_1": extract_rak_1,
    # #                 "sc_1": extract_sc_1,
    # #                 "sc_2": extract_sc_2,
    # #                 "sib_1": extract_sib_1,
    # #                 "sib_2": extract_sib_2
    #     }    
    
    def __check_pdf_metadata(self, pdf_bytes):
        # Define the lists of good and bad creator and producer strings
        creator_lst_good = [
            "3-Heights(TM) Image to PDF Converter",
            "PDFium",
            "pdfmake",
            "Style Report",
            "Crystal Reports",
            "JasperReports",
            "wkhtmltopdf 0.12.6",
            "wkhtmltopdf",
            "Ricoh Americas Corporation",
            "BIRT Report Engine .",
            "BIRT Engine",
            "FreeFlow VI eCompose 15.0.3.0",
            "iECCM, using PDF Engine.",
            "STANDARD CHARTERED BANK",
            "PDFKit",
            "Octagon 5.0",
            "alrajhi bank",
            "GMC Software AG",
            "HP Exstream Version",
            "OpenText Exstream Version",
            "vDOX"
        ]

        creator_lst_bad = [
            "Chrome",
            "Mozilla",
            "Microsoft",
            "Safari",
            "Samsung",
            "Acrobat",
            "macOS",
        ]

        producer_lst_good = [
            "iText",
            "pdfmake",
            "Ricoh Americas Corporation",
            "Powered By Crystal",
            "Style Report",
            "Qt 4.8.7"
        ]

        producer_lst_bad = [
            "3-Heights(TM)",
            "pdfium",
            "ilovePDF",
            "iOS",
            "macOS Version 13.3.1 (a) (Build 22E772610a) Quartz PDFContext",
            "OpenPDF 1.3.26",
            "PSPDFKit",
            "Tango+ PDF Writer",
            "excel",
            "canva",
            "word"
        ]


        try:
            pdfinfo_output = subprocess.Popen(["pdfinfo", "-"], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
            # Pass the PDF content to pdfinfo
            pdfinfo_output, pdfinfo_error = pdfinfo_output.communicate(input=pdf_bytes)

            if pdfinfo_output.returncode != 0:
                return False, "Error running pdfinfo: " + pdfinfo_error.strip()
        except FileNotFoundError:
            return False, "pdfinfo command not found. Make sure it's installed and in your PATH."

        # Extract the creator and producer strings, created_at, and modified_at from the pdfinfo output
        creator_string = None
        producer_string = None
        created_at = None
        modified_at = None

        for line in pdfinfo_output.splitlines():
            if line.startswith("Creator:"):
                creator_string = line[len("Creator:"):].strip()
            elif line.startswith("Producer:"):
                producer_string = line[len("Producer:"):].strip()
            elif line.startswith("CreationDate:"):
                created_at = line[len("CreationDate:"):].strip()
            elif line.startswith("ModDate:"):
                modified_at = line[len("ModDate:"):].strip()

        # Check conditions based on the presence of strings in the lists and created/modified timestamps
        if creator_string in creator_lst_good and producer_string in producer_lst_good:
            return True, None
        elif creator_string in creator_lst_good and producer_string in producer_lst_bad:
            return False, "Upload a non-edited PDF"
        elif creator_string in creator_lst_bad and producer_string in producer_lst_good:
            return False, "Upload a non-edited PDF"
        elif creator_string in creator_lst_bad and producer_string in producer_lst_bad:
            return False, "Upload a non-edited PDF"
        elif creator_string not in creator_lst_good + creator_lst_bad and producer_string in producer_lst_good:
            if created_at == modified_at:
                return True, None
            else:
                return False, "Upload a non-edited PDF"
        elif creator_string not in creator_lst_good + creator_lst_bad and producer_string in producer_lst_bad:
            return False, "Upload a non-edited PDF"
        elif creator_string in creator_lst_good and producer_string not in producer_lst_good + producer_lst_bad:
            if created_at == modified_at:
                return True, None
            else:
                return False, "Upload a non-edited PDF"
        elif creator_string in creator_lst_bad and producer_string not in producer_lst_good + producer_lst_bad:
            return False, "Upload a non-edited PDF"
        else:
            if created_at == modified_at:
                return True, None
            else:
                return False, "Upload a non-edited PDF"

        print("Creator:", creator_string)
        print("Producer:", producer_string)
        print("CreationDate:", created_at)
        print("ModDate:", modified_at)    

    # Function to convert PDF to images using pdf2image library
    def convert_pdf_to_images(self, pdf_content_stream):
        try:
            images = convert_from_bytes(pdf_content_stream.read(), dpi=300)
            return images
        except Exception as e:
            return []
    
    def __process_pdf_and_detect_labels(self, pdf_content, confidence_threshold=0.85):
        try:
            # Convert the PDF content bytes into a readable stream
            pdf_content_stream = BytesIO(pdf_content)

            # Convert the first page of the PDF to an image
            images = self.convert_pdf_to_images(pdf_content_stream)

            if images:
                # Convert PIL image to a bytes-like object
                img_byte_array = BytesIO()
                images[0].save(img_byte_array, format='JPEG')
                img_byte_array.seek(0)

                # Load the YOLOv5 model
                model_path = pkg_resources.resource_filename('bankstatementextractor', 'models/best.pt')
                model = torch.hub.load('ultralytics/yolov5', 'custom', path=model_path)

                # Load the image from the bytes-like object
                img = Image.open(img_byte_array)

                # Perform inference
                results = model(img)

                # Get bounding box coordinates, confidence scores, and labels
                boxes = results.pred[0][:, :4].cpu().numpy()
                confidences = results.pred[0][:, 4].cpu().numpy()
                labels = results.pred[0][:, 5].cpu().numpy().astype(int)

                detected_labels = []

                # Filter detections based on confidence threshold and collect labels
                for box, confidence, label in zip(boxes, confidences, labels):
                    if confidence >= confidence_threshold:
                        label_name = model.names[label]
                        detected_labels.append(label_name)

                return detected_labels
            else:
                return []
        except Exception as e:
            return []
            
            
    def extract(self, pdf_bytes):
        data=[]
        # Check PDF metadata
        metadata_result, metadata_message = self.__check_pdf_metadata(pdf_bytes)
        if metadata_result:
            # Process PDF and detect labels
            detected_labels = self.__process_pdf_and_detect_labels(pdf_bytes)
            if detected_labels:
                print("Detected Labels:", detected_labels)
                banks = Banks()
                for label in detected_labels:
                    if label in self.bank_labels:
                        data.append(getattr(banks, label)(pdf_bytes))
                        # bank_labels[label](file_path)  # Call the corresponding function with the file_path
                    else:
                        print(f"No function found for label: {label}")
            else:
                print("No labels detected.")
        else:
            print("Metadata Check Failed:", metadata_message)
        
        return data
        



from setuptools import setup, find_packages

with open("/Users/anjalimallissery/Downloads/Anjali_BS_Extract/requirements.txt") as f:
    required = f.read().splitlines()

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="bankstatementextractor",
    version="1.1.2",
    author="Anjali",
    author_email="anjalimenon217@gmail.com",
    description=("This repository contains a Python program designed to extract Optical Character Recognition (OCR)"
                 " data from bank statements, detect income and classify expenses"),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.7",
    install_requires=required,
    include_package_data=True,
)
